/*!
 * Copyright (C) 2006 by egnite Software GmbH. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 */

/*!
 * $Log$
 */

#include <cfg/os.h>
#include <cfg/clock.h>
#include <dev/board.h>

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#include <io.h>

#include <sys/version.h>
#include <sys/confnet.h>
#include <sys/atom.h>
#include <sys/heap.h>
#include <sys/thread.h>
#include <sys/timer.h>
#include <sys/event.h>
#include <sys/socket.h>

#include <hxmp3/mp3dec.h>

#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <net/route.h>
#include <pro/dhcp.h>

#include "config.h"


STATIONCONF *station;
RADIOCONTROL radio;

/*!
 * \brief Configure a station list entry.
 *
 * \param idx Index of the entry.
 * \param url URL of the station.
 *
 * \return 0 on success, -1 otherwise.
 */
int ConfigStation(u_char idx, CONST char *symbol, CONST char * url)
{
    u_long ip;
    u_short port = 80;
    char *buf;
    char *cp;

    if (idx >= MAXNUM_STATIONS) {
        idx = 0;
        while (idx < MAXNUM_STATIONS && station[idx].rs_port)
            idx++;
    }
    if (idx >= MAXNUM_STATIONS)
        return -1;
    else {
        buf = malloc(strlen(url) + 1);

        /* Extract IP address. */
        cp = buf;
        while (*url && *url != '/' && *url != ':')
            *cp++ = *url++;
        *cp = 0;
        if ((int) (ip = inet_addr(buf)) == -1)
            idx = -1;
        else {
            /* Extract URL path. */
            cp = buf;
            if (*url == '/') {
                url++;
                while (*url && *url != ':')
                    *cp++ = *url++;
            }
            *cp = 0;

            /* Extract port. */
            if (*url == ':')
                port = atoi(url + 1);

            if (port) {
                station[idx].rs_ip = ip;
                station[idx].rs_port = port;
                if (*buf) {
                    station[idx].rs_url = malloc(strlen(buf) + 1);
                    strcpy(station[idx].rs_url, buf);
                }
            }
        }

        free(buf);
    }
    return idx;
}

/*
 * Allocate heap memory for configuration structure.
 */
static void ConfigCreate(void)
{
    u_char idx;

    if (station == 0)
        station = malloc(MAXNUM_STATIONS * sizeof(STATIONCONF));
    else {
        /* Release all memory occupied by the current configuration. */
        for (idx = 0; idx < MAXNUM_STATIONS; idx++) {
            if (station[idx].rs_port == 0)
                break;
            if (station[idx].rs_url)
                free(station[idx].rs_url);
        }
    }
    memset(station, 0, MAXNUM_STATIONS * sizeof(STATIONCONF));
}

/*!
 * \brief Reset configuration.
 */
void ConfigResetFactory(void)
{
    ConfigCreate();

    /* Initial radio control settings. */
    radio.rc_rstation = 2;
    radio.rc_rvolume = 223;

    /* 
     * Add pre-configured radio stations. 
     */
#if 0
    /* Low bandwidth. */
    ConfigStation(MAXNUM_STATIONS, "Test4", "66.225.205.60:80");
    ConfigStation(MAXNUM_STATIONS, "Test5", "207.245.67.204:80");
    ConfigStation(MAXNUM_STATIONS, "Test6", "62.75.220.155:10022");
    ConfigStation(MAXNUM_STATIONS, "Test7", "208.45.137.232:8008");
    ConfigStation(MAXNUM_STATIONS, "Test8", "195.151.242.137:8000");
#else
    /* www.frequence3.fr */
    ConfigStation(MAXNUM_STATIONS, "FREQUENCE3", "88.191.22.199:8000");
    ConfigStation(MAXNUM_STATIONS, "FREQUENCE3", "194.158.114.68:8000");
    ConfigStation(MAXNUM_STATIONS, "FREQUENCE3", "194.158.114.67:8000");
    ConfigStation(MAXNUM_STATIONS, "FREQUENCE3", "193.251.154.243:8000");
    ConfigStation(MAXNUM_STATIONS, "FREQUENCE3", "213.186.56.206:8000");
    ConfigStation(MAXNUM_STATIONS, "FREQUENCE3", "194.158.114.66:8000");
    ConfigStation(MAXNUM_STATIONS, "FREQUENCE3", "213.186.41.160:8000");
    ConfigStation(MAXNUM_STATIONS, "FREQUENCE3", "88.191.11.19:8000");
    ConfigStation(MAXNUM_STATIONS, "FREQUENCE3", "62.4.21.91:8000");
    ConfigStation(MAXNUM_STATIONS, "FREQUENCE3", "212.129.62.140:8000");
    ConfigStation(MAXNUM_STATIONS, "FREQUENCE3", "194.117.194.66:8000");
    ConfigStation(MAXNUM_STATIONS, "FREQUENCE3", "213.186.37.124:8000");
    ConfigStation(MAXNUM_STATIONS, "FREQUENCE3", "80.65.234.120:8000");
    ConfigStation(MAXNUM_STATIONS, "FREQUENCE3", "213.161.201.21:8000");
    ConfigStation(MAXNUM_STATIONS, "FREQUENCE3", "213.186.60.54:8000");
    ConfigStation(MAXNUM_STATIONS, "FREQUENCE3", "193.251.154.242:8000");
    ConfigStation(MAXNUM_STATIONS, "FREQUENCE3", "83.143.18.76:8000");
    ConfigStation(MAXNUM_STATIONS, "FREQUENCE3", "193.17.192.35:8000");
    ConfigStation(MAXNUM_STATIONS, "FREQUENCE3", "193.222.128.135:8000");

    ConfigStation(MAXNUM_STATIONS, "TEST1", "64.236.34.97:80/stream/1005");
    ConfigStation(MAXNUM_STATIONS, "TEST2", "207.200.96.230:8040");
    ConfigStation(MAXNUM_STATIONS, "TEST3", "194.158.114.68:8000");
    ConfigStation(MAXNUM_STATIONS, "TEST4", "88.191.34.99:7020");
#endif
}

